package com.samelement.iot_notification.appwidget;

import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.widget.RemoteViews;

import androidx.core.graphics.ColorUtils;

import com.samelement.iot_notification.R;
import com.samelement.iot_notification.appwidget.db.AppWidgetObject;
import com.samelement.iot_notification.appwidget.db.WidgetReaderDbHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

/**
 * Implementation of App Widget functionality.
 * App Widget Configuration implemented in {@link SimpleTextWidgetConfigureActivity SimpleTextWidgetConfigureActivity}
 */
public class SimpleTextWidget extends AppWidgetProvider {
    static void updateAppWidget(Context context, AppWidgetManager appWidgetManager,
                                int appWidgetId) {
        WidgetReaderDbHelper dbHelper = WidgetReaderDbHelper.getInstance(context);
        List<AppWidgetObject> appWidgetObjects = dbHelper.getById(appWidgetId + "");

        // Construct the RemoteViews object
        RemoteViews views = getRemoteViews(context, appWidgetManager, appWidgetId);

        if (appWidgetObjects.size() > 0) {
            boolean isDark = appWidgetObjects.get(0).getColor().equals("dark");
            int alpha = appWidgetObjects.get(0).getTransparency();
            int alphaContent;
            int alphaTitle;
            if (isDark) {
                alphaTitle = ColorUtils.setAlphaComponent(context.getResources().getColor(R.color.colorContentDark), alpha);
                alphaContent = ColorUtils.setAlphaComponent(context.getResources().getColor(R.color.colorContentDark), alpha);
            } else {
                alphaTitle = ColorUtils.setAlphaComponent(context.getResources().getColor(R.color.colorContentLight), alpha);
                alphaContent = ColorUtils.setAlphaComponent(context.getResources().getColor(R.color.colorContentLight), alpha);
            }

            views.setInt(R.id.title_layout, "setBackgroundColor", alphaTitle);
            views.setInt(R.id.item_1, "setBackgroundColor", alphaContent);
            views.setInt(R.id.item_2, "setBackgroundColor", alphaContent);
            views.setInt(R.id.item_3, "setBackgroundColor", alphaContent);
            views.setInt(R.id.item_4, "setBackgroundColor", alphaContent);

            // title
            views.setTextViewText(R.id.appwidget_title, appWidgetObjects.get(0).getTitle());

            if (appWidgetObjects.size() >= 3) {
                views.setViewVisibility(R.id.section_below, View.VISIBLE);
            }

            // resource with unit Temp / C
            if (appWidgetObjects.size() >= 1) {
                String resourceName = appWidgetObjects.get(0).getResource();
                String unit = appWidgetObjects.get(0).getUnit();
                if (unit != null && unit.length() > 0 && !unit.equals("null")) {
                    resourceName = resourceName + " / " + unit;
                }

                views.setTextViewText(R.id.appwidget_resource_with_unit_1, resourceName);
                views.setTextViewText(R.id.appwidget_datetime_1,
                        getDate(appWidgetObjects.get(0).getDatetime()));
                views.setTextViewText(R.id.appwidget_value_1, appWidgetObjects.get(0).getValue());
            }

            if (appWidgetObjects.size() >= 2) {
                views.setViewVisibility(R.id.item_2, View.VISIBLE);

                String resourceName = appWidgetObjects.get(1).getResource();
                String unit = appWidgetObjects.get(1).getUnit();
                if (unit != null && unit.length() > 0 && !unit.equals("null")) {
                    resourceName = resourceName + " / " + unit;
                }

                views.setTextViewText(R.id.appwidget_resource_with_unit_2, resourceName);
                views.setTextViewText(R.id.appwidget_datetime_2,
                        getDate(appWidgetObjects.get(1).getDatetime()));
                views.setTextViewText(R.id.appwidget_value_2, appWidgetObjects.get(1).getValue());
            }

            if (appWidgetObjects.size() >= 3) {
                views.setViewVisibility(R.id.item_3, View.VISIBLE);

                String resourceName = appWidgetObjects.get(2).getResource();
                String unit = appWidgetObjects.get(2).getUnit();
                if (unit != null && unit.length() > 0 && !unit.equals("null")) {
                    resourceName = resourceName + " / " + unit;
                }

                views.setTextViewText(R.id.appwidget_resource_with_unit_3, resourceName);
                views.setTextViewText(R.id.appwidget_datetime_3,
                        getDate(appWidgetObjects.get(2).getDatetime()));
                views.setTextViewText(R.id.appwidget_value_3, appWidgetObjects.get(2).getValue());
            }

            if (appWidgetObjects.size() >= 4) {
                views.setViewVisibility(R.id.item_4, View.VISIBLE);

                String resourceName = appWidgetObjects.get(3).getResource();
                String unit = appWidgetObjects.get(3).getUnit();
                if (unit != null && unit.length() > 0 && !unit.equals("null")) {
                    resourceName = resourceName + " / " + unit;
                }

                views.setTextViewText(R.id.appwidget_resource_with_unit_4, resourceName);
                views.setTextViewText(R.id.appwidget_datetime_4,
                        getDate(appWidgetObjects.get(3).getDatetime()));
                views.setTextViewText(R.id.appwidget_value_4, appWidgetObjects.get(3).getValue());
            }
        }
        // Instruct the widget manager to update the widget
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }


    static SimpleDateFormat dbFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);
    static SimpleDateFormat uiFormatter = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss", Locale.US);

    static String getDate(String date) {
        if (date != null) {
            try {
                Date currentDate = dbFormatter.parse(date);
                if (currentDate != null) {
                    return uiFormatter.format(currentDate);
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        return date;
    }

    @Override
    public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
        // There may be multiple widgets active, so update all of them
        for (int appWidgetId : appWidgetIds) {
            updateAppWidget(context, appWidgetManager, appWidgetId);
        }
    }

    @Override
    public void onAppWidgetOptionsChanged(Context context, AppWidgetManager appWidgetManager, int appWidgetId, Bundle newOptions) {
        // Obtain appropriate widget and update it.
        updateAppWidget(context, appWidgetManager, appWidgetId);

        super.onAppWidgetOptionsChanged(context, appWidgetManager, appWidgetId, newOptions);
    }

    /**
     * Determine appropriate view based on width provided.
     */
    private static RemoteViews getRemoteViews(Context context, AppWidgetManager appWidgetManager, int appWidgetId) {
        // See the dimensions and
        Bundle options = appWidgetManager.getAppWidgetOptions(appWidgetId);

        // Get min width and height.
        int minHeight = options
                .getInt(AppWidgetManager.OPTION_APPWIDGET_MIN_HEIGHT);


        // First find out rows and columns based on width provided.
        int rows = getCellsForSize(minHeight);

        switch (rows) {
            case 2:
                return new RemoteViews(context.getPackageName(),
                        R.layout.simple_text_widget_2);
            case 3:
                return new RemoteViews(context.getPackageName(),
                        R.layout.simple_text_widget_3);
            case 4:
                return new RemoteViews(context.getPackageName(),
                        R.layout.simple_text_widget_4);
            default:
                return new RemoteViews(context.getPackageName(),
                        R.layout.simple_text_widget);
        }
    }

    /**
     * Returns number of cells needed for given size of the widget.
     *
     * @param size Widget size in dp.
     * @return Size in number of cells.
     */
    private static int getCellsForSize(int size) {
        int n = 2;
        while (70 * n - 30 < size) {
            ++n;
        }
        return n - 1;
    }
}

